@@ -3,15 +3,16 @@
 
 #pragma once
 
-#include <QFont>
 #include <QObject>
 #include <QQuickTextDocument>
 #include <QTextCursor>
-#include <QUrl>
+
+#include "completionmodel.h"
+#include "userlistmodel.h"
 
 class QTextDocument;
 class NeoChatRoom;
-class Controller;
+class SyntaxHighlighter;
 
 class ChatDocumentHandler : public QObject
 {
@@ -21,18 +22,11 @@ class ChatDocumentHandler : public QObject
     Q_PROPERTY(int selectionStart READ selectionStart WRITE setSelectionStart NOTIFY selectionStartChanged)
     Q_PROPERTY(int selectionEnd READ selectionEnd WRITE setSelectionEnd NOTIFY selectionEndChanged)
 
-    Q_PROPERTY(NeoChatRoom *room READ room WRITE setRoom NOTIFY roomChanged)
+    Q_PROPERTY(CompletionModel *completionModel READ completionModel NOTIFY completionModelChanged)
+
+    Q_PROPERTY(NeoChatRoom *room READ room NOTIFY roomChanged)
 
 public:
-    enum AutoCompletionType {
-        User,
-        Emoji,
-        Command,
-        None,
-        Ignore,
-    };
-    Q_ENUM(AutoCompletionType)
-
     explicit ChatDocumentHandler(QObject *parent = nullptr);
 
     [[nodiscard]] QQuickTextDocument *document() const;
@@ -50,35 +44,33 @@ public:
     [[nodiscard]] NeoChatRoom *room() const;
     void setRoom(NeoChatRoom *room);
 
-    /// This function will look at the current QTextCursor and determine if there
-    /// is the possibility to autocomplete it.
-    Q_INVOKABLE QVariantMap getAutocompletionInfo(bool isAutocompleting);
-    Q_INVOKABLE void replaceAutoComplete(const QString &word);
+    Q_INVOKABLE void complete(int index);
 
+    void updateCompletions();
+    CompletionModel *completionModel() const;
 Q_SIGNALS:
     void documentChanged();
     void cursorPositionChanged();
+    void roomChanged();
+    void completionModelChanged();
     void selectionStartChanged();
     void selectionEndChanged();
-    void roomChanged();
-    void joinRoom(QString roomName);
 
 private:
-    [[nodiscard]] QTextCursor textCursor() const;
-    [[nodiscard]] QTextDocument *textDocument() const;
+    int completionStartIndex() const;
 
     QQuickTextDocument *m_document;
 
-    NeoChatRoom *m_room;
+    NeoChatRoom *m_room = nullptr;
+    bool completionVisible = false;
 
     int m_cursorPosition;
     int m_selectionStart;
     int m_selectionEnd;
 
-    int m_autoCompleteBeginPosition = -1;
-    int m_autoCompleteEndPosition = -1;
+    SyntaxHighlighter *m_highlighter = nullptr;
 
-    QString m_lastState;
+    CompletionModel::AutoCompletionType m_completionType = CompletionModel::None;
+
+    CompletionModel *m_completionModel = nullptr;
 };
-
-Q_DECLARE_METATYPE(ChatDocumentHandler::AutoCompletionType);
