@@ -4,12 +4,14 @@
 #include "clipboard.h"
 
 #include <QClipboard>
+#include <QDateTime>
 #include <QDir>
 #include <QFileInfo>
 #include <QGuiApplication>
 #include <QImage>
 #include <QMimeData>
 #include <QRegularExpression>
+#include <QStandardPaths>
 #include <QUrl>
 
 Clipboard::Clipboard(QObject *parent)
@@ -29,27 +31,34 @@ QImage Clipboard::image() const
     return m_clipboard->image();
 }
 
-bool Clipboard::saveImage(const QUrl &localPath) const
+QString Clipboard::saveImage(QString localPath) const
 {
-    if (!localPath.isLocalFile()) {
-        return false;
+    if (!QDir().exists(QStringLiteral("%1/screenshots").arg(QStandardPaths::writableLocation(QStandardPaths::CacheLocation)))) {
+        QDir().mkdir(QStringLiteral("%1/screenshots").arg(QStandardPaths::writableLocation(QStandardPaths::CacheLocation)));
+    }
+    if (localPath.isEmpty()) {
+        localPath = QStringLiteral("file://%1/screenshots/%2.png")
+                        .arg(QStandardPaths::writableLocation(QStandardPaths::CacheLocation),
+                             QDateTime::currentDateTime().toString(QStringLiteral("yyyy-MM-dd-hh-mm-ss")));
+    }
+    QUrl url(localPath);
+    if (!url.isLocalFile()) {
+        return {};
+    }
+    auto image = this->image();
+
+    if (image.isNull()) {
+        return {};
     }
 
-    auto i = image();
-
-    if (i.isNull()) {
-        return false;
-    }
-
-    QString path = QFileInfo(localPath.toLocalFile()).absolutePath();
     QDir dir;
-    if (!dir.exists(path)) {
-        dir.mkpath(path);
+    if (!dir.exists(localPath)) {
+        dir.mkpath(localPath);
     }
 
-    i.save(localPath.toLocalFile());
+    image.save(url.toLocalFile());
 
-    return true;
+    return localPath;
 }
 
 void Clipboard::saveText(QString message)
