@@ -14,23 +14,17 @@
 #include <KWindowEffects>
 #endif
 
-#include <QAuthenticator>
-#include <QClipboard>
 #include <QFile>
 #include <QFileInfo>
 #include <QGuiApplication>
-#include <QMovie>
-#include <QNetworkReply>
-#include <QPixmap>
+#include <QImageReader>
+#include <QNetworkProxy>
 #include <QQuickItem>
 #include <QQuickTextDocument>
 #include <QQuickWindow>
 #include <QStandardPaths>
 #include <QStringBuilder>
-#include <QSysInfo>
-#include <QTextDocument>
 #include <QTimer>
-#include <utility>
 
 #include <signal.h>
 
@@ -40,24 +34,24 @@
 #include "neochataccountregistry.h"
 #endif
 
-#include <csapi/account-data.h>
+#include <connection.h>
 #include <csapi/content-repo.h>
-#include <csapi/joining.h>
 #include <csapi/logout.h>
 #include <csapi/profile.h>
-#include <events/eventcontent.h>
 #include <qt_connection_util.h>
-#include <settings.h>
+
+#ifdef QUOTIENT_07
+#include <csapi/notifications.h>
+#include <eventstats.h>
+#endif
 
 #include "neochatconfig.h"
 #include "neochatroom.h"
 #include "neochatuser.h"
+#include "notificationsmanager.h"
 #include "roommanager.h"
-#include "utils.h"
 #include "windowcontroller.h"
 
-#include <KStandardShortcut>
-
 #if defined(Q_OS_WIN) || defined(Q_OS_MAC)
 #include "trayicon.h"
 #elif !defined(Q_OS_ANDROID)
@@ -72,6 +66,8 @@ Controller::Controller(QObject *parent)
     Connection::setRoomType<NeoChatRoom>();
     Connection::setUserType<NeoChatUser>();
 
+    setApplicationProxy();
+
 #ifndef Q_OS_ANDROID
     if (NeoChatConfig::self()->systemTray()) {
         m_trayIcon = new TrayIcon(this);
@@ -124,8 +120,109 @@ Controller::Controller(QObject *parent)
         sigaction(sig, &sa, nullptr);
     }
 #endif
+
+    connect(&AccountRegistry::instance(), &AccountRegistry::accountCountChanged, this, &Controller::activeConnectionIndexChanged);
+
+#ifdef QUOTIENT_07
+    static int oldAccountCount = 0;
+    connect(&AccountRegistry::instance(), &AccountRegistry::accountCountChanged, this, [=]() {
+        if (AccountRegistry::instance().size() > oldAccountCount) {
+            auto connection = AccountRegistry::instance().accounts()[AccountRegistry::instance().size() - 1];
+            connect(connection, &Connection::syncDone, this, [=]() {
+                bool changes = false;
+                for (const auto &room : connection->allRooms()) {
+                    if (m_notificationCounts[room] != room->unreadStats().notableCount) {
+                        m_notificationCounts[room] = room->unreadStats().notableCount;
+                        changes = true;
+                    }
+                }
+                if (changes) {
+                    handleNotifications();
+                }
+            });
+        }
+        oldAccountCount = AccountRegistry::instance().size();
+    });
+#endif
 }
 
+#ifdef QUOTIENT_07
+void Controller::handleNotifications()
+{
+    static bool initial = true;
+    static QStringList oldNotifications;
+    if (!m_connection) {
+        return;
+    }
+    auto job = m_connection->callApi<GetNotificationsJob>();
+
+    connect(job, &BaseJob::success, this, [this, job]() {
+        const auto notifications = job->jsonData()["notifications"].toArray();
+        if (initial) {
+            initial = false;
+            for (const auto &n : notifications) {
+                oldNotifications += n.toObject()["event"].toObject()["event_id"].toString();
+            }
+            return;
+        }
+        for (const auto &n : notifications) {
+            const auto notification = n.toObject();
+            if (notification["read"].toBool()) {
+                oldNotifications.removeOne(notification["event"].toObject()["event_id"].toString());
+                continue;
+            }
+            if (oldNotifications.contains(notification["event"].toObject()["event_id"].toString())) {
+                continue;
+            }
+            oldNotifications += notification["event"].toObject()["event_id"].toString();
+            auto room = m_connection->room(notification["room_id"].toString());
+
+            // If room exists, room is NOT active OR the application is NOT active, show notification
+            if (room && !(room->id() == RoomManager::instance().currentRoom()->id() && QGuiApplication::applicationState() == Qt::ApplicationActive)) {
+                // The room might have been deleted (for example rejected invitation).
+                auto sender = room->user(notification["event"].toObject()["sender"].toString());
+
+                QString body;
+
+                if (notification["event"].toObject()["type"].toString() == "org.matrix.msc3381.poll.start") {
+                    body = notification["event"]
+                               .toObject()["content"]
+                               .toObject()["org.matrix.msc3381.poll.start"]
+                               .toObject()["question"]
+                               .toObject()["body"]
+                               .toString();
+                } else {
+                    body = notification["event"].toObject()["content"].toObject()["body"].toString();
+                }
+
+                if (notification["event"]["type"] == "m.room.encrypted") {
+#ifdef Quotient_E2EE_ENABLED
+                    auto decrypted = m_connection->decryptNotification(notification);
+                    body = decrypted["content"].toObject()["body"].toString();
+#endif
+                    if (body.isEmpty()) {
+                        body = i18n("Encrypted Message");
+                    }
+                }
+
+                QImage avatar_image;
+                if (!sender->avatarUrl(room).isEmpty()) {
+                    avatar_image = sender->avatar(128, room);
+                } else {
+                    avatar_image = room->avatar(128);
+                }
+                NotificationsManager::instance().postNotification(dynamic_cast<NeoChatRoom *>(room),
+                                                                  sender->displayname(room),
+                                                                  body,
+                                                                  avatar_image,
+                                                                  notification["event"].toObject()["event_id"].toString(),
+                                                                  true);
+            }
+        }
+    });
+}
+#endif
+
 Controller &Controller::instance()
 {
     static Controller _instance;
@@ -137,13 +234,6 @@ void Controller::showWindow()
     WindowController::instance().showAndRaiseWindow(QString());
 }
 
-inline QString accessTokenFileName(const AccountSettings &account)
-{
-    QString fileName = account.userId();
-    fileName.replace(':', '_');
-    return QStandardPaths::writableLocation(QStandardPaths::AppLocalDataLocation) + '/' + fileName;
-}
-
 void Controller::loginWithAccessToken(const QString &serverAddr, const QString &user, const QString &token, const QString &deviceName)
 {
     if (user.isEmpty() || token.isEmpty()) {
@@ -184,7 +274,6 @@ void Controller::logout(Connection *conn, bool serverSideLogout)
     }
 
     SettingsGroup("Accounts").remove(conn->userId());
-    QFile(accessTokenFileName(AccountSettings(conn->userId()))).remove();
 
     QKeychain::DeletePasswordJob job(qAppName());
     job.setAutoDelete(true);
@@ -194,8 +283,11 @@ void Controller::logout(Connection *conn, bool serverSideLogout)
     job.start();
     loop.exec();
 
-    if (conn == activeConnection() && AccountRegistry::instance().count() > 1) {
-        setActiveConnection(AccountRegistry::instance().accounts()[0]);
+    if (AccountRegistry::instance().count() > 1) {
+        // Only set the connection if the the account being logged out is currently active
+        if (conn == activeConnection()) {
+            setActiveConnection(AccountRegistry::instance().accounts()[0]);
+        }
     } else {
         setActiveConnection(nullptr);
     }
@@ -274,15 +366,7 @@ void Controller::invokeLogin()
                 if (accessTokenLoadingJob->error() == QKeychain::Error::NoError) {
                     accessToken = accessTokenLoadingJob->binaryData();
                 } else {
-                    // No access token from the keychain, try token file
-                    // TODO FIXME this code is racy since the file might have
-                    // already been removed. But since the other code do a blocking
-                    // dbus call, the probability are not high that it will happen.
-                    // loadAccessTokenFromFile is also mostly legacy nowadays
-                    accessToken = loadAccessTokenFromFile(account);
-                    if (accessToken.isEmpty()) {
-                        return;
-                    }
+                    return;
                 }
 
                 auto connection = new Connection(account.homeserver());
@@ -316,21 +400,6 @@ void Controller::invokeLogin()
     }
 }
 
-QByteArray Controller::loadAccessTokenFromFile(const AccountSettings &account)
-{
-    QFile accountTokenFile{accessTokenFileName(account)};
-    if (accountTokenFile.open(QFile::ReadOnly)) {
-        if (accountTokenFile.size() < 1024) {
-            return accountTokenFile.readAll();
-        }
-
-        qWarning() << "File" << accountTokenFile.fileName() << "is" << accountTokenFile.size() << "bytes long - too long for a token, ignoring it.";
-    }
-    qWarning() << "Could not open access token file" << accountTokenFile.fileName();
-
-    return {};
-}
-
 QKeychain::ReadPasswordJob *Controller::loadAccessTokenFromKeyChain(const AccountSettings &account)
 {
     qDebug() << "Reading access token from the keychain for" << account.userId();
@@ -342,24 +411,6 @@ QKeychain::ReadPasswordJob *Controller::loadAccessTokenFromKeyChain(const Accoun
         if (job->error() == QKeychain::Error::NoError) {
             return;
         }
-        if (job->error() == QKeychain::Error::EntryNotFound) {
-            // no access token from the keychain, try token file
-            auto accessToken = loadAccessTokenFromFile(account);
-            if (!accessToken.isEmpty()) {
-                qDebug() << "Migrating the access token from file to the keychain for " << account.userId();
-                bool removed = false;
-                bool saved = saveAccessTokenToKeyChain(account, accessToken);
-                if (saved) {
-                    QFile accountTokenFile{accessTokenFileName(account)};
-                    removed = accountTokenFile.remove();
-                }
-                if (!(saved && removed)) {
-                    qDebug() << "Migrating the access token from the file to the keychain "
-                                "failed";
-                }
-                return;
-            }
-        }
 
         switch (job->error()) {
         case QKeychain::EntryNotFound:
@@ -384,22 +435,6 @@ QKeychain::ReadPasswordJob *Controller::loadAccessTokenFromKeyChain(const Accoun
     return job;
 }
 
-bool Controller::saveAccessTokenToFile(const AccountSettings &account, const QByteArray &accessToken)
-{
-    // (Re-)Make a dedicated file for access_token.
-    QFile accountTokenFile{accessTokenFileName(account)};
-    accountTokenFile.remove(); // Just in case
-
-    auto fileDir = QFileInfo(accountTokenFile).dir();
-    if (!((fileDir.exists() || fileDir.mkpath(".")) && accountTokenFile.open(QFile::WriteOnly))) {
-        Q_EMIT errorOccured("I/O Denied: Cannot save access token.");
-    } else {
-        accountTokenFile.write(accessToken);
-        return true;
-    }
-    return false;
-}
-
 bool Controller::saveAccessTokenToKeyChain(const AccountSettings &account, const QByteArray &accessToken)
 {
     qDebug() << "Save the access token to the keychain for " << account.userId();
@@ -414,7 +449,7 @@ bool Controller::saveAccessTokenToKeyChain(const AccountSettings &account, const
 
     if (job.error()) {
         qWarning() << "Could not save access token to the keychain: " << qPrintable(job.errorString());
-        return saveAccessTokenToFile(account, accessToken);
+        return false;
     }
     return true;
 }
@@ -441,17 +476,6 @@ void Controller::markAllMessagesAsRead(Connection *conn)
     }
 }
 
-void Controller::setAboutData(const KAboutData &aboutData)
-{
-    m_aboutData = aboutData;
-    Q_EMIT aboutDataChanged();
-}
-
-KAboutData Controller::aboutData() const
-{
-    return m_aboutData;
-}
-
 bool Controller::supportSystemTray() const
 {
 #ifdef Q_OS_ANDROID
@@ -569,7 +593,7 @@ void Controller::setActiveConnection(Connection *connection)
         return;
     }
     if (m_connection != nullptr) {
-        disconnect(connection, &Connection::syncError, this, nullptr);
+        disconnect(m_connection, &Connection::syncError, this, nullptr);
     }
     m_connection = connection;
     if (connection != nullptr) {
@@ -593,6 +617,7 @@ void Controller::setActiveConnection(Connection *connection)
     }
     NeoChatConfig::self()->save();
     Q_EMIT activeConnectionChanged();
+    Q_EMIT activeConnectionIndexChanged();
 }
 
 void Controller::saveWindowGeometry()
@@ -642,8 +667,11 @@ void Controller::openOrCreateDirectChat(NeoChatUser *user)
     const auto existing = activeConnection()->directChats();
 
     if (existing.contains(user)) {
-        RoomManager::instance().enterRoom(static_cast<NeoChatRoom *>(activeConnection()->room(existing.value(user))));
-        return;
+        const auto &room = static_cast<NeoChatRoom *>(activeConnection()->room(existing.value(user)));
+        if (room) {
+            RoomManager::instance().enterRoom(room);
+            return;
+        }
     }
     activeConnection()->requestDirectChat(user);
 }
@@ -697,3 +725,71 @@ bool Controller::encryptionSupported() const
     return false;
 #endif
 }
+
+void Controller::forceRefreshTextDocument(QQuickTextDocument *textDocument, QQuickItem *item)
+{
+    // HACK: Workaround bug QTBUG 93281
+    connect(textDocument->textDocument(), SIGNAL(imagesLoaded()), item, SLOT(updateWholeDocument()));
+}
+
+void Controller::setApplicationProxy()
+{
+    NeoChatConfig *cfg = NeoChatConfig::self();
+    QNetworkProxy proxy;
+
+    // type match to ProxyType from neochatconfig.kcfg
+    switch (cfg->proxyType()) {
+    case 1: // HTTP
+        proxy.setType(QNetworkProxy::HttpProxy);
+        proxy.setHostName(cfg->proxyHost());
+        proxy.setPort(cfg->proxyPort());
+        break;
+    case 2: // SOCKS 5
+        proxy.setType(QNetworkProxy::Socks5Proxy);
+        proxy.setHostName(cfg->proxyHost());
+        proxy.setPort(cfg->proxyPort());
+        break;
+    case 0: // System Default
+    default:
+        // do nothing
+        break;
+    }
+
+    QNetworkProxy::setApplicationProxy(proxy);
+}
+
+int Controller::activeConnectionIndex() const
+{
+#ifdef QUOTIENT_07
+    auto result = std::find_if(Accounts.accounts().begin(), Accounts.accounts().end(), [this](const auto &it) {
+        return it == m_connection;
+    });
+    return result - Accounts.accounts().begin();
+#else
+    for (int i = 0; i < AccountRegistry::instance().rowCount(); i++) {
+        if (AccountRegistry::instance().data(AccountRegistry::instance().index(i, 0), AccountRegistry::UserIdRole).toString() == m_connection->userId()) {
+            return i;
+        }
+    }
+    return 0;
+#endif
+}
+
+int Controller::quotientMinorVersion() const
+{
+// TODO libQuotient 0.7: Replace with version function from libQuotient
+#ifdef QUOTIENT_07
+    return 7;
+#else
+    return 6;
+#endif
+}
+
+bool Controller::isFlatpak() const
+{
+#ifdef NEOCHAT_FLATPAK
+    return true;
+#else
+    return false;
+#endif
+}
