@@ -6,16 +6,10 @@
 #include <QObject>
 #include <QQuickItem>
 
-#include <KAboutData>
 #include <KFormat>
 
-class QKeySequences;
-
-#include <connection.h>
-#include <csapi/list_public_rooms.h>
-#include <room.h>
+#include <jobs/basejob.h>
 #include <settings.h>
-#include <user.h>
 
 class NeoChatRoom;
 class NeoChatUser;
@@ -23,34 +17,40 @@ class TrayIcon;
 class QWindow;
 class QQuickTextDocument;
 
+namespace Quotient
+{
+class Connection;
+class Room;
+}
+
 namespace QKeychain
 {
 class ReadPasswordJob;
 }
 
-using namespace Quotient;
-
 class Controller : public QObject
 {
     Q_OBJECT
     Q_PROPERTY(int accountCount READ accountCount NOTIFY accountCountChanged)
     Q_PROPERTY(bool quitOnLastWindowClosed READ quitOnLastWindowClosed WRITE setQuitOnLastWindowClosed NOTIFY quitOnLastWindowClosedChanged)
-    Q_PROPERTY(Connection *activeConnection READ activeConnection WRITE setActiveConnection NOTIFY activeConnectionChanged)
+    Q_PROPERTY(Quotient::Connection *activeConnection READ activeConnection WRITE setActiveConnection NOTIFY activeConnectionChanged)
     Q_PROPERTY(bool busy READ busy WRITE setBusy NOTIFY busyChanged)
-    Q_PROPERTY(KAboutData aboutData READ aboutData WRITE setAboutData NOTIFY aboutDataChanged)
     Q_PROPERTY(bool supportSystemTray READ supportSystemTray CONSTANT)
     Q_PROPERTY(bool hasWindowSystem READ hasWindowSystem CONSTANT)
     Q_PROPERTY(bool isOnline READ isOnline NOTIFY isOnlineChanged)
     Q_PROPERTY(bool encryptionSupported READ encryptionSupported CONSTANT)
+    Q_PROPERTY(int activeConnectionIndex READ activeConnectionIndex NOTIFY activeConnectionIndexChanged)
+    Q_PROPERTY(int quotientMinorVersion READ quotientMinorVersion CONSTANT)
+    Q_PROPERTY(bool isFlatpak READ isFlatpak CONSTANT)
 
 public:
     static Controller &instance();
 
-    void setActiveConnection(Connection *connection);
-    [[nodiscard]] Connection *activeConnection() const;
+    void setActiveConnection(Quotient::Connection *connection);
+    [[nodiscard]] Quotient::Connection *activeConnection() const;
 
-    void addConnection(Connection *c);
-    void dropConnection(Connection *c);
+    void addConnection(Quotient::Connection *c);
+    void dropConnection(Quotient::Connection *c);
 
     Q_INVOKABLE void loginWithAccessToken(const QString &, const QString &, const QString &, const QString &);
 
@@ -66,13 +66,11 @@ public:
     [[nodiscard]] bool busy() const;
     void setBusy(bool busy);
 
-    void setAboutData(const KAboutData &aboutData);
-    [[nodiscard]] KAboutData aboutData() const;
-
     [[nodiscard]] bool supportSystemTray() const;
 
-    bool saveAccessTokenToFile(const AccountSettings &account, const QByteArray &accessToken);
-    bool saveAccessTokenToKeyChain(const AccountSettings &account, const QByteArray &accessToken);
+    bool saveAccessTokenToKeyChain(const Quotient::AccountSettings &account, const QByteArray &accessToken);
+
+    int activeConnectionIndex() const;
 
     enum PasswordStatus {
         Success,
@@ -98,22 +96,31 @@ public:
     Q_INVOKABLE QString plainText(QQuickTextDocument *document) const;
     bool encryptionSupported() const;
 
+    Q_INVOKABLE void forceRefreshTextDocument(QQuickTextDocument *textDocument, QQuickItem *item);
+
+    Q_INVOKABLE void setApplicationProxy();
+
+    int quotientMinorVersion() const;
+    bool isFlatpak() const;
+
 private:
     explicit Controller(QObject *parent = nullptr);
 
-    QPointer<Connection> m_connection;
+    QPointer<Quotient::Connection> m_connection;
     bool m_busy = false;
     TrayIcon *m_trayIcon = nullptr;
 
-    static QByteArray loadAccessTokenFromFile(const AccountSettings &account);
-    QKeychain::ReadPasswordJob *loadAccessTokenFromKeyChain(const AccountSettings &account);
+    QKeychain::ReadPasswordJob *loadAccessTokenFromKeyChain(const Quotient::AccountSettings &account);
 
     void loadSettings();
     void saveSettings() const;
     bool m_isOnline = true;
+    QMap<Quotient::Room *, int> m_notificationCounts;
 
-    KAboutData m_aboutData;
     bool hasWindowSystem() const;
+#ifdef QUOTIENT_07
+    void handleNotifications();
+#endif
 
 private Q_SLOTS:
     void invokeLogin();
@@ -135,15 +142,15 @@ Q_SIGNALS:
     void quitOnLastWindowClosedChanged();
     void unreadCountChanged();
     void activeConnectionChanged();
-    void aboutDataChanged();
     void passwordStatus(Controller::PasswordStatus _t1);
     void userConsentRequired(QUrl url);
     void testConnectionResult(const QString &connection, bool usable);
     void isOnlineChanged(bool isOnline);
-    void keyVerificationRequest(int timeLeft, Connection *connection, const QString &transactionId, const QString &deviceId);
+    void keyVerificationRequest(int timeLeft, Quotient::Connection *connection, const QString &transactionId, const QString &deviceId);
     void keyVerificationStart();
     void keyVerificationAccept(const QString &commitment);
     void keyVerificationKey(const QString &sas);
+    void activeConnectionIndexChanged();
 
 public Q_SLOTS:
     void logout(Quotient::Connection *conn, bool serverSideLogout);
@@ -153,14 +160,14 @@ public Q_SLOTS:
 };
 
 // TODO libQuotient 0.7: Drop
-class NeochatChangePasswordJob : public BaseJob
+class NeochatChangePasswordJob : public Quotient::BaseJob
 {
 public:
-    explicit NeochatChangePasswordJob(const QString &newPassword, bool logoutDevices, const Omittable<QJsonObject> &auth = none);
+    explicit NeochatChangePasswordJob(const QString &newPassword, bool logoutDevices, const Quotient::Omittable<QJsonObject> &auth = Quotient::none);
 };
 
-class NeochatDeleteDeviceJob : public BaseJob
+class NeochatDeleteDeviceJob : public Quotient::BaseJob
 {
 public:
-    explicit NeochatDeleteDeviceJob(const QString &deviceId, const Omittable<QJsonObject> &auth = none);
+    explicit NeochatDeleteDeviceJob(const QString &deviceId, const Quotient::Omittable<QJsonObject> &auth = Quotient::none);
 };
