@@ -3,6 +3,12 @@
 
 #include "login.h"
 
+#ifdef QUOTIENT_07
+#include <accountregistry.h>
+#else
+#include "neochataccountregistry.h"
+#endif
+
 #include <connection.h>
 #include <qt_connection_util.h>
 
@@ -12,6 +18,8 @@
 
 #include <KLocalizedString>
 
+using namespace Quotient;
+
 Login::Login(QObject *parent)
     : QObject(parent)
 {
@@ -37,6 +45,12 @@ void Login::init()
             return;
         }
 
+        m_isLoggedIn = AccountRegistry::instance().isLoggedIn(m_matrixId);
+        Q_EMIT isLoggedInChanged();
+        if (m_isLoggedIn) {
+            return;
+        }
+
         m_testing = true;
         Q_EMIT testingChanged();
         if (!m_connection) {
@@ -142,7 +156,10 @@ void Login::login()
     m_isLoggingIn = true;
     Q_EMIT isLoggingInChanged();
 
-    m_connection->loginWithPassword(m_matrixId, m_password, m_deviceName, QString());
+    // Some servers do not have a .well_known file. So we login via the username part from the mxid,
+    // rather than with the full mxid, as that would lead to an invalid user.
+    auto username = m_matrixId.mid(1, m_matrixId.indexOf(":") - 1);
+    m_connection->loginWithPassword(username, m_password, m_deviceName, QString());
 }
 
 bool Login::supportsPassword() const
@@ -179,3 +196,8 @@ bool Login::isLoggingIn() const
 {
     return m_isLoggingIn;
 }
+
+bool Login::isLoggedIn() const
+{
+    return m_isLoggedIn;
+}
