@@ -2,12 +2,7 @@
 // SPDX-License-Identifier: GPL-3.0-only
 
 #include <QCommandLineParser>
-#include <QDebug>
-#include <QFontDatabase>
-#include <QGuiApplication>
 #include <QIcon>
-#include <QNetworkAccessManager>
-#include <QNetworkProxy>
 #include <QNetworkProxyFactory>
 #include <QQmlApplicationEngine>
 #include <QQmlContext>
@@ -27,11 +22,9 @@
 #endif
 #ifdef HAVE_WINDOWSYSTEM
 #include <KWindowSystem>
-#include <kwindowsystem_version.h>
 #endif
 #include <KLocalizedContext>
 #include <KLocalizedString>
-#include <KWindowConfig>
 
 #include "neochat-version.h"
 
@@ -41,18 +34,15 @@
 #include "neochataccountregistry.h"
 #endif
 
-#include <csapi/joining.h>
-#include <csapi/leaving.h>
 #include <networkaccessmanager.h>
 #include <room.h>
+#include <util.h>
 
 #include "actionshandler.h"
 #include "blurhashimageprovider.h"
-#include "chatboxhelper.h"
 #include "chatdocumenthandler.h"
 #include "clipboard.h"
 #include "collapsestateproxymodel.h"
-#include "commandmodel.h"
 #include "controller.h"
 #include "customemojimodel.h"
 #include "devicesmodel.h"
@@ -60,6 +50,7 @@
 #include "filetypesingleton.h"
 #include "joinrulesevent.h"
 #include "linkpreviewer.h"
+#include "keywordnotificationrulemodel.h"
 #include "login.h"
 #include "matriximageprovider.h"
 #include "messageeventmodel.h"
@@ -68,9 +59,13 @@
 #include "neochatroom.h"
 #include "neochatuser.h"
 #include "notificationsmanager.h"
+#ifdef QUOTIENT_07
+#include "pollhandler.h"
+#endif
 #include "publicroomlistmodel.h"
 #include "roomlistmodel.h"
 #include "roommanager.h"
+#include "serverlistmodel.h"
 #include "sortfilterroomlistmodel.h"
 #include "sortfilterspacelistmodel.h"
 #include "spacehierarchycache.h"
@@ -85,6 +80,9 @@
 #ifdef HAVE_COLORSCHEME
 #include "colorschemer.h"
 #endif
+#include "completionmodel.h"
+#include "neochatuser.h"
+#include "statemodel.h"
 
 #ifdef HAVE_RUNNER
 #include "runner.h"
@@ -137,8 +135,9 @@ int main(int argc, char *argv[])
     }
 
     QApplication::setStyle(QStringLiteral("breeze"));
-    auto font = app.font();
+    QFont font(QStringLiteral("Segoe UI Emoji"));
     font.setPointSize(10);
+    font.setHintingPreference(QFont::PreferNoHinting);
     app.setFont(font);
 #endif
     KLocalizedString::setApplicationDomain("neochat");
@@ -151,14 +150,24 @@ int main(int argc, char *argv[])
                      i18n("Matrix client"),
                      KAboutLicense::GPL_V3,
                      i18n("© 2018-2020 Black Hat, 2020-2022 KDE Community"));
-    about.addAuthor(i18n("Black Hat"), QString(), QStringLiteral("bhat@encom.eu.org"));
-    about.addAuthor(i18n("Carl Schwan"), QString(), QStringLiteral("carl@carlschwan.eu"));
-    about.addAuthor(i18n("Tobias Fella"), QString(), QStringLiteral("fella@posteo.de"));
+    about.addAuthor(i18n("Carl Schwan"), i18n("Maintainer"), QStringLiteral("carl@carlschwan.eu"), QStringLiteral("https://carlschwan.eu"));
+    about.addAuthor(i18n("Tobias Fella"), i18n("Maintainer"), QStringLiteral("fella@posteo.de"), QStringLiteral("https://tobiasfella.de"));
+    about.addAuthor(i18n("James Graham"), i18n("Maintainer"), QStringLiteral("james.h.graham@protonmail.com"));
+    about.addCredit(i18n("Black Hat"), i18n("Original author of Spectral"), QStringLiteral("bhat@encom.eu.org"));
+    about.addCredit(i18n("Alexey Rusakov"), i18n("Maintainer of Quotient"), QStringLiteral("Kitsune-Ral@users.sf.net"));
+    about.setTranslator(i18nc("NAME OF TRANSLATORS", "Your names"), i18nc("EMAIL OF TRANSLATORS", "Your emails"));
     about.setOrganizationDomain("kde.org");
 
     about.addComponent(QStringLiteral("libQuotient"),
                        i18n("A Qt5 library to write cross-platform clients for Matrix"),
-                       QString(),
+#ifdef QUOTIENT_07
+                       i18nc("<version number> (built against <possibly different version number>)",
+                             "%1 (built against %2)",
+                             Quotient::versionString(),
+                             QStringLiteral(Quotient_VERSION_STRING)),
+#else
+                       QStringLiteral(QUOTIENT_VERSION),
+#endif
                        QStringLiteral("https://github.com/quotient-im/libquotient"),
                        KAboutLicense::LGPL_V2_1);
 
@@ -176,9 +185,7 @@ int main(int argc, char *argv[])
     FileTypeSingleton fileTypeSingleton;
 
     Login *login = new Login();
-    ChatBoxHelper chatBoxHelper;
     UrlHelper urlHelper;
-    SpaceHierarchyCache spaceHierarchyCache;
 
 #ifdef HAVE_COLORSCHEME
     ColorSchemer colorScheme;
@@ -196,30 +203,41 @@ int main(int argc, char *argv[])
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "FileType", &fileTypeSingleton);
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "LoginHelper", login);
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "UrlHelper", &urlHelper);
-    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "EmojiModel", new EmojiModel(&app));
-    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "CommandModel", new CommandModel(&app));
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "EmojiModel", &EmojiModel::instance());
+#ifdef QUOTIENT_07
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "AccountRegistry", &Quotient::Accounts);
+#else
     qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "AccountRegistry", &Quotient::AccountRegistry::instance());
-    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "SpaceHierarchyCache", &spaceHierarchyCache);
+#endif
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "SpaceHierarchyCache", &SpaceHierarchyCache::instance());
+    qmlRegisterSingletonInstance("org.kde.neochat", 1, 0, "CustomEmojiModel", &CustomEmojiModel::instance());
     qmlRegisterType<ActionsHandler>("org.kde.neochat", 1, 0, "ActionsHandler");
-    qmlRegisterType<ChatBoxHelper>("org.kde.neochat", 1, 0, "ChatBoxHelper");
     qmlRegisterType<ChatDocumentHandler>("org.kde.neochat", 1, 0, "ChatDocumentHandler");
     qmlRegisterType<RoomListModel>("org.kde.neochat", 1, 0, "RoomListModel");
     qmlRegisterType<KWebShortcutModel>("org.kde.neochat", 1, 0, "WebShortcutModel");
     qmlRegisterType<UserListModel>("org.kde.neochat", 1, 0, "UserListModel");
-    qmlRegisterType<CustomEmojiModel>("org.kde.neochat", 1, 0, "CustomEmojiModel");
     qmlRegisterType<MessageEventModel>("org.kde.neochat", 1, 0, "MessageEventModel");
     qmlRegisterType<CollapseStateProxyModel>("org.kde.neochat", 1, 0, "CollapseStateProxyModel");
     qmlRegisterType<MessageFilterModel>("org.kde.neochat", 1, 0, "MessageFilterModel");
     qmlRegisterType<PublicRoomListModel>("org.kde.neochat", 1, 0, "PublicRoomListModel");
     qmlRegisterType<UserDirectoryListModel>("org.kde.neochat", 1, 0, "UserDirectoryListModel");
+    qmlRegisterType<ServerListModel>("org.kde.neochat", 1, 0, "ServerListModel");
     qmlRegisterType<SortFilterRoomListModel>("org.kde.neochat", 1, 0, "SortFilterRoomListModel");
     qmlRegisterType<SortFilterSpaceListModel>("org.kde.neochat", 1, 0, "SortFilterSpaceListModel");
     qmlRegisterType<DevicesModel>("org.kde.neochat", 1, 0, "DevicesModel");
     qmlRegisterType<LinkPreviewer>("org.kde.neochat", 1, 0, "LinkPreviewer");
+    qmlRegisterType<CompletionModel>("org.kde.neochat", 1, 0, "CompletionModel");
+    qmlRegisterType<StateModel>("org.kde.neochat", 1, 0, "StateModel");
+#ifdef QUOTIENT_07
+    qmlRegisterType<PollHandler>("org.kde.neochat", 1, 0, "PollHandler");
+#endif
+    qmlRegisterType<KeywordNotificationRuleModel>("org.kde.neochat", 1, 0, "KeywordNotificationRuleModel");
     qmlRegisterUncreatableType<RoomMessageEvent>("org.kde.neochat", 1, 0, "RoomMessageEvent", "ENUM");
     qmlRegisterUncreatableType<PushNotificationState>("org.kde.neochat", 1, 0, "PushNotificationState", "ENUM");
+    qmlRegisterUncreatableType<PushNotificationAction>("org.kde.neochat", 1, 0, "PushNotificationAction", "ENUM");
     qmlRegisterUncreatableType<NeoChatRoomType>("org.kde.neochat", 1, 0, "NeoChatRoomType", "ENUM");
     qmlRegisterUncreatableType<UserType>("org.kde.neochat", 1, 0, "UserType", "ENUM");
+    qmlRegisterUncreatableType<NeoChatUser>("org.kde.neochat", 1, 0, "NeoChatUser", {});
 
     qRegisterMetaType<User *>("User*");
     qRegisterMetaType<User *>("const User*");
@@ -238,6 +256,9 @@ int main(int argc, char *argv[])
     qRegisterMetaType<QVector<EmojiEntry>>("QVector<EmojiEntry>");
 #endif
 #endif
+    qmlRegisterSingletonType("org.kde.neochat", 1, 0, "About", [](QQmlEngine *engine, QJSEngine *) -> QJSValue {
+        return engine->toScriptValue(KAboutData::applicationData());
+    });
 
 #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     qRegisterMetaTypeStreamOperators<Emoji>();
@@ -250,19 +271,16 @@ int main(int argc, char *argv[])
 
     QCommandLineParser parser;
     parser.setApplicationDescription(i18n("Client for the matrix communication protocol"));
-    parser.addPositionalArgument(QStringLiteral("urls"), i18n("Supports appstream: url scheme"));
+    parser.addPositionalArgument(QStringLiteral("urls"), i18n("Supports matrix: url scheme"));
 
     about.setupCommandLine(&parser);
     parser.process(app);
     about.processCommandLine(&parser);
 
-    Controller::instance().setAboutData(about);
-
-    engine.addImportPath("qrc:/imports");
     engine.addImageProvider(QLatin1String("mxc"), new MatrixImageProvider);
     engine.addImageProvider(QLatin1String("blurhash"), new BlurhashImageProvider);
 
-    engine.load(QUrl(QStringLiteral("qrc:/qml/main.qml")));
+    engine.load(QUrl(QStringLiteral("qrc:/main.qml")));
     if (engine.rootObjects().isEmpty()) {
         return -1;
     }
