@@ -13,6 +13,9 @@
 #include <events/simplestateevents.h>
 #include <user.h>
 
+#ifdef QUOTIENT_07
+#include "pollevent.h"
+#endif
 #include "stickerevent.h"
 
 #include <QDebug>
@@ -23,8 +26,11 @@
 #include <KFormat>
 #include <KLocalizedString>
 
+#include "neochatuser.h"
 #include "utils.h"
 
+using namespace Quotient;
+
 QHash<int, QByteArray> MessageEventModel::roleNames() const
 {
     QHash<int, QByteArray> roles = QAbstractItemModel::roleNames();
@@ -56,6 +62,11 @@ QHash<int, QByteArray> MessageEventModel::roleNames() const
     roles[AuthorIdRole] = "authorId";
     roles[MediaUrlRole] = "mediaUrl";
     roles[VerifiedRole] = "verified";
+    roles[DisplayNameForInitialsRole] = "displayNameForInitials";
+    roles[AuthorDisplayNameRole] = "authorDisplayName";
+    roles[IsNameChangeRole] = "isNameChange";
+    roles[IsAvatarChangeRole] = "isAvatarChange";
+    roles[IsRedactedRole] = "isRedacted";
     return roles;
 }
 
@@ -92,7 +103,11 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
         if (m_currentRoom->timelineSize() < 10 && !room->allHistoryLoaded()) {
             room->getPreviousContent(50);
         }
+#ifdef QUOTIENT_07
+        lastReadEventId = room->lastFullyReadEventId();
+#else
         lastReadEventId = room->readMarkerEventId();
+#endif
 
         using namespace Quotient;
         connect(m_currentRoom, &Room::aboutToAddNewMessages, this, [this](RoomEventsRange events) {
@@ -141,7 +156,11 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
             endInsertRows();
             if (!m_lastReadEventIndex.isValid()) {
                 // no read marker, so see if we need to create one.
+#ifdef QUOTIENT_07
+                moveReadMarker(m_currentRoom->lastFullyReadEventId());
+#else
                 moveReadMarker(m_currentRoom->readMarkerEventId());
+#endif
             }
             if (biggest < m_currentRoom->maxTimelineIndex()) {
                 auto rowBelowInserted = m_currentRoom->maxTimelineIndex() - biggest + timelineBaseIndex() - 1;
@@ -286,6 +305,9 @@ int MessageEventModel::refreshEventRoles(const QString &id, const QVector<int> &
             return -1;
         }
         row = int(timelineIt - m_currentRoom->messageEvents().rbegin()) + timelineBaseIndex();
+        if (data(index(row, 0), EventTypeRole).toInt() == ReadMarker || data(index(row, 0), EventTypeRole).toInt() == Other) {
+            row++;
+        }
     }
     refreshEventRoles(row, roles);
     return row;
@@ -415,7 +437,7 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     if (m_lastReadEventIndex.row() == row) {
         switch (role) {
         case EventTypeRole:
-            return QStringLiteral("readMarker");
+            return DelegateType::ReadMarker;
         case TimeRole: {
             const QDateTime eventDate = data(index(m_lastReadEventIndex.row() + 1, 0), TimeRole).toDateTime();
             const KFormat format;
@@ -462,35 +484,43 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         if (auto e = eventCast<const RoomMessageEvent>(&evt)) {
             switch (e->msgtype()) {
             case MessageEventType::Emote:
-                return "emote";
+                return DelegateType::Emote;
             case MessageEventType::Notice:
-                return "notice";
+                return DelegateType::Notice;
             case MessageEventType::Image:
-                return "image";
+                return DelegateType::Image;
             case MessageEventType::Audio:
-                return "audio";
+                return DelegateType::Audio;
             case MessageEventType::Video:
-                return "video";
+                return DelegateType::Video;
             default:
                 break;
             }
             if (e->hasFileContent()) {
-                return "file";
+                return DelegateType::File;
             }
 
-            return "message";
+            return DelegateType::Message;
         }
         if (is<const StickerEvent>(evt)) {
-            return "sticker";
+            return DelegateType::Sticker;
         }
         if (evt.isStateEvent()) {
-            return "state";
+            return DelegateType::State;
         }
         if (is<const EncryptedEvent>(evt)) {
-            return "encrypted";
+            return DelegateType::Encrypted;
         }
+#ifdef QUOTIENT_07
+        if (is<PollStartEvent>(evt)) {
+            if (evt.isRedacted()) {
+                return DelegateType::Message;
+            }
+            return DelegateType::Poll;
+        }
+#endif
 
-        return "other";
+        return DelegateType::Other;
     }
 
     if (role == EventResolvedTypeRole) {
@@ -526,6 +556,7 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         if (auto e = eventCast<const StickerEvent>(&evt)) {
             return QVariant::fromValue(e->image().originalJson);
         }
+        return evt.contentJson();
     }
 
     if (role == HighlightRole) {
@@ -666,36 +697,36 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
             return {};
         }
 
-        QString type;
+        DelegateType type;
         if (auto e = eventCast<const RoomMessageEvent>(replyPtr)) {
             switch (e->msgtype()) {
             case MessageEventType::Emote:
-                type = "emote";
+                type = DelegateType::Emote;
                 break;
             case MessageEventType::Notice:
-                type = "notice";
+                type = DelegateType::Notice;
                 break;
             case MessageEventType::Image:
-                type = "image";
+                type = DelegateType::Image;
                 break;
             case MessageEventType::Audio:
-                type = "audio";
+                type = DelegateType::Audio;
                 break;
             case MessageEventType::Video:
-                type = "video";
+                type = DelegateType::Video;
                 break;
             default:
                 if (e->hasFileContent()) {
-                    type = "file";
+                    type = DelegateType::File;
                     break;
                 }
-                type = "message";
+                type = DelegateType::Message;
             }
 
         } else if (is<const StickerEvent>(*replyPtr)) {
-            type = "sticker";
+            type = DelegateType::Sticker;
         } else {
-            type = "other";
+            type = DelegateType::Other;
         }
 
         QVariant content;
@@ -720,8 +751,11 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     if (role == ShowAuthorRole) {
         for (auto r = row + 1; r < rowCount(); ++r) {
             auto i = index(r);
-            if (data(i, SpecialMarksRole) != EventStatus::Hidden) {
-                return data(i, AuthorRole) != data(idx, AuthorRole) || data(i, EventTypeRole) == "state"
+            // Note !itemData(i).empty() is a check for instances where rows have been removed, e.g. when the read marker is moved.
+            // While the row is removed the subsequent row indexes are not changed so we need to skip over the removed index.
+            // See - https://doc.qt.io/qt-5/qabstractitemmodel.html#beginRemoveRows
+            if (data(i, SpecialMarksRole) != EventStatus::Hidden && !itemData(i).empty()) {
+                return data(i, AuthorRole) != data(idx, AuthorRole) || data(i, EventTypeRole) == MessageEventModel::State
                     || data(i, TimeRole).toDateTime().msecsTo(data(idx, TimeRole).toDateTime()) > 600000
                     || data(i, TimeRole).toDateTime().toLocalTime().date().day() != data(idx, TimeRole).toDateTime().toLocalTime().date().day();
             }
@@ -733,7 +767,10 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     if (role == ShowSectionRole) {
         for (auto r = row + 1; r < rowCount(); ++r) {
             auto i = index(r);
-            if (data(i, SpecialMarksRole) != EventStatus::Hidden) {
+            // Note !itemData(i).empty() is a check for instances where rows have been removed, e.g. when the read marker is moved.
+            // While the row is removed the subsequent row indexes are not changed so we need to skip over the removed index.
+            // See - https://doc.qt.io/qt-5/qabstractitemmodel.html#beginRemoveRows
+            if (data(i, SpecialMarksRole) != EventStatus::Hidden && !itemData(i).empty()) {
                 const auto day = data(idx, TimeRole).toDateTime().toLocalTime().date().dayOfYear();
                 const auto previousEventDay = data(i, TimeRole).toDateTime().toLocalTime().date().dayOfYear();
                 return day != previousEventDay;
@@ -818,11 +855,48 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         if (evt.originalEvent()) {
             auto encrypted = dynamic_cast<const EncryptedEvent *>(evt.originalEvent());
             Q_ASSERT(encrypted);
-            return m_currentRoom->connection()->isVerifiedSession(encrypted->sessionId());
+            return m_currentRoom->connection()->isVerifiedSession(encrypted->sessionId().toLatin1());
+        }
+#endif
+#endif
+        return false;
+    }
+
+    if (role == DisplayNameForInitialsRole) {
+        auto user = static_cast<NeoChatUser *>(isPending ? m_currentRoom->localUser() : m_currentRoom->user(evt.senderId()));
+        return user->displayname(m_currentRoom).remove(QStringLiteral(" (%1)").arg(user->id()));
+    }
+
+    if (role == AuthorDisplayNameRole) {
+        if (is<RoomMemberEvent>(evt) && !evt.unsignedJson()["prev_content"]["displayname"].isNull() && evt.stateKey() == evt.senderId()) {
+            auto previousDisplayName = evt.unsignedJson()["prev_content"]["displayname"].toString().toHtmlEscaped();
+            if (previousDisplayName.isEmpty()) {
+                previousDisplayName = evt.senderId();
+            }
+            return previousDisplayName;
+        } else {
+            auto author = static_cast<NeoChatUser *>(isPending ? m_currentRoom->localUser() : m_currentRoom->user(evt.senderId()));
+            return m_currentRoom->htmlSafeMemberName(author->id());
+        }
+    }
+
+    if (role == IsNameChangeRole) {
+        auto roomMemberEvent = eventCast<const RoomMemberEvent>(&evt);
+        if (roomMemberEvent) {
+            return roomMemberEvent->isRename();
         }
         return false;
-#endif
-#endif
+    }
+
+    if (role == IsAvatarChangeRole) {
+        auto roomMemberEvent = eventCast<const RoomMemberEvent>(&evt);
+        if (roomMemberEvent) {
+            return roomMemberEvent->isAvatarUpdate();
+        }
+        return false;
+    }
+    if (role == IsRedactedRole) {
+        return evt.isRedacted();
     }
 
     return {};
