@@ -3,53 +3,58 @@
 
 #include "neochatroom.h"
 
-#include <cmark.h>
-
 #include <QFileInfo>
 #include <QMetaObject>
 #include <QMimeDatabase>
+#include <QTemporaryFile>
 #include <QTextDocument>
 #include <functional>
 
-#if QT_VERSION > QT_VERSION_CHECK(6, 0, 0)
 #include <QMediaMetaData>
 #include <QMediaPlayer>
-#endif
 
 #include <qcoro/qcorosignal.h>
 #include <qcoro/task.h>
 
 #include <connection.h>
-#include <csapi/account-data.h>
-#include <csapi/content-repo.h>
-#include <csapi/leaving.h>
+#include <csapi/directory.h>
 #include <csapi/pushrules.h>
 #include <csapi/redaction.h>
+#include <csapi/report_content.h>
 #include <csapi/room_state.h>
-#include <csapi/rooms.h>
 #include <csapi/typing.h>
-#include <events/accountdataevents.h>
-#include <events/eventcontent.h>
+#include <events/encryptionevent.h>
 #include <events/reactionevent.h>
+#include <events/redactionevent.h>
+#include <events/roomavatarevent.h>
 #include <events/roomcanonicalaliasevent.h>
-#include <events/roomcreateevent.h>
-#include <events/roommessageevent.h>
+#include <events/roommemberevent.h>
 #include <events/roompowerlevelsevent.h>
-#include <events/typingevent.h>
+#include <events/simplestateevents.h>
 #include <jobs/downloadfilejob.h>
 #include <qt_connection_util.h>
-#include <user.h>
-#include <csapi/report_content.h>
-#include <events/eventcontent.h>
 
 #include "controller.h"
+#include "joinrulesevent.h"
 #include "neochatconfig.h"
+#include "neochatuser.h"
 #include "notificationsmanager.h"
+#ifdef QUOTIENT_07
+#include "pollevent.h"
+#include "pollhandler.h"
+#endif
+#include "filetransferpseudojob.h"
 #include "stickerevent.h"
 #include "utils.h"
 
+#ifndef Q_OS_ANDROID
+#include <KIO/Job>
+#endif
+#include <KJobTrackerInterface>
 #include <KLocalizedString>
 
+using namespace Quotient;
+
 NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinState)
     : Room(connection, std::move(roomId), joinState)
 {
@@ -73,6 +78,7 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
     connect(this, &Room::displaynameChanged, this, &NeoChatRoom::displayNameChanged);
 
     connectSingleShot(this, &Room::baseStateLoaded, this, [this]() {
+        Q_EMIT canEncryptRoomChanged();
         if (this->joinState() != JoinState::Invite) {
             return;
         }
@@ -86,6 +92,9 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
         }
         NotificationsManager::instance().postInviteNotification(this, htmlSafeDisplayName(), htmlSafeMemberName(senderId), avatar_image);
     });
+    connect(this, &Room::changed, this, [this] {
+        Q_EMIT canEncryptRoomChanged();
+    });
 }
 
 void NeoChatRoom::uploadFile(const QUrl &url, const QString &body)
@@ -99,24 +108,32 @@ QCoro::Task<void> NeoChatRoom::doUploadFile(QUrl url, QString body)
         co_return;
     }
 
-#if QT_VERSION > QT_VERSION_CHECK(6, 0, 0)
     auto mime = QMimeDatabase().mimeTypeForUrl(url);
     QFileInfo fileInfo(url.toLocalFile());
     EventContent::TypedBase *content;
     if (mime.name().startsWith("image/")) {
-        QImage image;
+        QImage image(url.toLocalFile());
         content = new EventContent::ImageContent(url, fileInfo.size(), mime, image.size(), fileInfo.fileName());
     } else if (mime.name().startsWith("audio/")) {
         content = new EventContent::AudioContent(url, fileInfo.size(), mime, fileInfo.fileName());
     } else if (mime.name().startsWith("video/")) {
         QMediaPlayer player;
+#if QT_VERSION > QT_VERSION_CHECK(6, 0, 0)
         player.setSource(url);
+#else
+        player.setMedia(url);
+#endif
         co_await qCoro(&player, &QMediaPlayer::mediaStatusChanged);
+#if QT_VERSION > QT_VERSION_CHECK(6, 0, 0)
         auto resolution = player.metaData().value(QMediaMetaData::Resolution).toSize();
+#else
+        auto resolution = player.metaData(QMediaMetaData::Resolution).toSize();
+#endif
         content = new EventContent::VideoContent(url, fileInfo.size(), mime, resolution, fileInfo.fileName());
     } else {
         content = new EventContent::FileContent(url, fileInfo.size(), mime, fileInfo.fileName());
     }
+#ifdef QUOTIENT_07
     QString txnId = postFile(body.isEmpty() ? url.fileName() : body, content);
 #else
     QString txnId = postFile(body.isEmpty() ? url.fileName() : body, url, false);
@@ -143,6 +160,14 @@ QCoro::Task<void> NeoChatRoom::doUploadFile(QUrl url, QString body)
             setFileUploadingProgress(int(float(progress) / float(total) * 100));
         }
     });
+#ifndef Q_OS_ANDROID
+    auto job = new FileTransferPseudoJob(FileTransferPseudoJob::Upload, url.toLocalFile(), txnId);
+    connect(this, &Room::fileTransferProgress, job, &FileTransferPseudoJob::fileTransferProgress);
+    connect(this, &Room::fileTransferCompleted, job, &FileTransferPseudoJob::fileTransferCompleted);
+    connect(this, &Room::fileTransferFailed, job, &FileTransferPseudoJob::fileTransferFailed);
+    KIO::getJobTracker()->registerJob(job);
+    job->start();
+#endif
 }
 
 void NeoChatRoom::acceptInvitation()
@@ -176,7 +201,7 @@ void NeoChatRoom::sendTypingNotification(bool isTyping)
     connection()->callApi<SetTypingJob>(BackgroundRequest, localUser()->id(), id(), isTyping, 10000);
 }
 
-const RoomMessageEvent *NeoChatRoom::lastEvent(bool ignoreStateEvent) const
+const RoomEvent *NeoChatRoom::lastEvent(bool ignoreStateEvent) const
 {
     for (auto timelineItem = messageEvents().rbegin(); timelineItem < messageEvents().rend(); timelineItem++) {
         const RoomEvent *event = timelineItem->get();
@@ -206,6 +231,11 @@ const RoomMessageEvent *NeoChatRoom::lastEvent(bool ignoreStateEvent) const
         if (auto lastEvent = eventCast<const RoomMessageEvent>(event)) {
             return lastEvent;
         }
+#ifdef QUOTIENT_07
+        if (auto lastEvent = eventCast<const PollStartEvent>(event)) {
+            return lastEvent;
+        }
+#endif
     }
     return nullptr;
 }
@@ -226,7 +256,7 @@ bool NeoChatRoom::lastEventIsSpoiler() const
 QString NeoChatRoom::lastEventToString() const
 {
     if (auto event = lastEvent()) {
-        return htmlSafeMemberName(event->senderId()) + (event->isStateEvent() ? " " : ": ") + eventToString(*event);
+        return roomMembername(event->senderId()) + (event->isStateEvent() ? " " : ": ") + eventToString(*event);
     }
     return QLatin1String("");
 }
@@ -297,29 +327,39 @@ QDateTime NeoChatRoom::lastActiveTime()
 
 QString NeoChatRoom::subtitleText()
 {
-    QString subtitle = this->lastEventToString().size() == 0 ? this->topic() : this->lastEventToString();
+    static const QRegularExpression blockquote("(\r\n\t|\n|\r\t|)> ");
+    static const QRegularExpression heading("(\r\n\t|\n|\r\t|)\\#{1,6} ");
+    static const QRegularExpression newlines("(\r\n\t|\n|\r\t)");
+    static const QRegularExpression bold1("(\\*\\*|__)(?=\\S)([^\\r]*\\S)\\1");
+    static const QRegularExpression bold2("(\\*|_)(?=\\S)([^\\r]*\\S)\\1");
+    static const QRegularExpression strike1("~~(.*)~~");
+    static const QRegularExpression strike2("~(.*)~");
+    static const QRegularExpression del("<del>(.*)</del>");
+    static const QRegularExpression multileLineCode("```([^```]+)```");
+    static const QRegularExpression singleLinecode("`([^`]+)`");
+    QString subtitle = lastEventToString().size() == 0 ? topic() : lastEventToString();
 
     subtitle
         // replace blockquote, i.e. '> text'
-        .replace(QRegularExpression("(\r\n\t|\n|\r\t|)> "), " ")
+        .replace(blockquote, " ")
         // replace headings, i.e. "# text"
-        .replace(QRegularExpression("(\r\n\t|\n|\r\t|)\\#{1,6} "), " ")
+        .replace(heading, " ")
         // replace newlines
-        .replace(QRegularExpression("(\r\n\t|\n|\r\t)"), " ")
+        .replace(newlines, " ")
         // replace '**text**' and '__text__'
-        .replace(QRegularExpression("(\\*\\*|__)(?=\\S)([^\\r]*\\S)\\1"), "\\2")
+        .replace(bold1, "\\2")
         // replace '*text*' and '_text_'
-        .replace(QRegularExpression("(\\*|_)(?=\\S)([^\\r]*\\S)\\1"), "\\2")
+        .replace(bold2, "\\2")
         // replace '~~text~~'
-        .replace(QRegularExpression("~~(.*)~~"), "\\1")
+        .replace(strike1, "\\1")
         // replace '~text~'
-        .replace(QRegularExpression("~(.*)~"), "\\1")
+        .replace(strike2, "\\1")
         // replace '<del>text</del>'
-        .replace(QRegularExpression("<del>(.*)</del>"), "\\1")
+        .replace(del, "\\1")
         // replace '```code```'
-        .replace(QRegularExpression("```([^```]+)```"), "\\1")
+        .replace(multileLineCode, "\\1")
         // replace '`code`'
-        .replace(QRegularExpression("`([^`]+)`"), "\\1");
+        .replace(singleLinecode, "\\1");
 
     return subtitle.size() > 0 ? subtitle : QStringLiteral(" ");
 }
@@ -473,16 +513,17 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
             // FIXME: Rewind to the name that was at the time of this event
             auto subjectName = this->htmlSafeMemberName(e.userId());
             if (e.membership() == MembershipType::Leave) {
-                auto displayName = e.prevContent()->displayName;
 #ifdef QUOTIENT_07
-                if (displayName) {
-                    subjectName = sanitized(*displayName).toHtmlEscaped();
+                if (e.prevContent() && e.prevContent()->displayName) {
+                    subjectName = sanitized(*e.prevContent()->displayName).toHtmlEscaped();
 #else
-                if (displayName.isEmpty()) {
-                    subjectName = sanitized(displayName).toHtmlEscaped();
+                if (e.prevContent() && e.prevContent()->displayName.isEmpty()) {
+                    subjectName = sanitized(e.prevContent()->displayName).toHtmlEscaped();
 #endif
                 }
             }
+            subjectName = QStringLiteral("<a href=\"https://matrix.to/#/%1\" style=\"color: %2\">%3</a>")
+                              .arg(e.userId(), static_cast<NeoChatUser *>(user(e.userId()))->color().name(), subjectName);
 
             // The below code assumes senderName output in AuthorRole
             switch (e.membership()) {
@@ -510,14 +551,14 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
                     return text;
                 }
                 // Part 2: profile changes of joined members
-                if (e.isRename() && NeoChatConfig::self()->showRename()) {
+                if (e.isRename()) {
                     if (e.displayName().isEmpty()) {
                         text = i18nc("their refers to a singular user", "cleared their display name");
                     } else {
                         text = i18nc("their refers to a singular user", "changed their display name to %1", e.displayName().toHtmlEscaped());
                     }
                 }
-                if (e.isAvatarUpdate() && NeoChatConfig::self()->showAvatarUpdate()) {
+                if (e.isAvatarUpdate()) {
                     if (!text.isEmpty()) {
                         text += i18n(" and ");
                     }
@@ -533,6 +574,9 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
                         text += i18nc("their refers to a singular user", "updated their avatar");
                     }
                 }
+                if (text.isEmpty()) {
+                    text = i18nc("<user> changed nothing", "changed nothing");
+                }
                 return text;
             }
             case MembershipType::Leave:
@@ -547,8 +591,15 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
                     ? i18n("has put %1 out of the room: %2", subjectName, e.contentJson()["reason"_ls].toString().toHtmlEscaped())
                     : i18n("left the room");
             case MembershipType::Ban:
-                return (e.senderId() != e.userId()) ? i18n("banned %1 from the room: %2", subjectName, e.contentJson()["reason"_ls].toString().toHtmlEscaped())
-                                                    : i18n("self-banned from the room");
+                if (e.senderId() != e.userId()) {
+                    if (e.reason().isEmpty()) {
+                        return i18n("banned %1 from the room", subjectName);
+                    } else {
+                        return i18n("banned %1 from the room: %2", subjectName, e.reason().toHtmlEscaped());
+                    }
+                } else {
+                    return i18n("self-banned from the room");
+                }
             case MembershipType::Knock:
                 return i18n("requested an invite");
             default:;
@@ -593,6 +644,11 @@ QString NeoChatRoom::eventToString(const RoomEvent &evt, Qt::TextFormat format,
             return e.stateKey().isEmpty() ? i18n("updated %1 state", e.matrixType())
                                           : i18n("updated %1 state for %2", e.matrixType(), e.stateKey().toHtmlEscaped());
         },
+#ifdef QUOTIENT_07
+        [](const PollStartEvent &e) {
+            return e.question();
+        },
+#endif
         i18n("Unknown event"));
 }
 
@@ -638,24 +694,6 @@ void NeoChatRoom::removeLocalAlias(const QString &alias)
     setLocalAliases(a);
 }
 
-QString NeoChatRoom::markdownToHTML(const QString &markdown)
-{
-    const auto str = markdown.toUtf8();
-    char *tmp_buf = cmark_markdown_to_html(str.constData(), str.size(), CMARK_OPT_HARDBREAKS);
-
-    const std::string html(tmp_buf);
-
-    free(tmp_buf);
-
-    auto result = QString::fromStdString(html).trimmed();
-
-    result.replace("<!-- raw HTML omitted -->", "");
-    result.replace("<p>", "");
-    result.replace("</p>", "");
-
-    return result;
-}
-
 QString msgTypeToString(MessageEventType msgType)
 {
     switch (msgType) {
@@ -680,11 +718,6 @@ QString msgTypeToString(MessageEventType msgType)
     }
 }
 
-QString NeoChatRoom::preprocessText(const QString &text)
-{
-    return markdownToHTML(text);
-}
-
 void NeoChatRoom::postMessage(const QString &rawText, const QString &text, MessageEventType type, const QString &replyEventId, const QString &relateToEventId)
 {
     postHtmlMessage(rawText, text, type, replyEventId, relateToEventId);
@@ -816,12 +849,24 @@ bool NeoChatRoom::canSendState(const QString &eventType) const
     auto pl = plEvent->powerLevelForState(eventType);
     auto currentPl = plEvent->powerLevelForUser(localUser()->id());
 
+#ifndef QUOTIENT_07
+    if (eventType == "m.room.history_visibility") {
+        return false;
+    } else {
+        return currentPl >= pl;
+    }
+#else
     return currentPl >= pl;
+#endif
 }
 
 bool NeoChatRoom::readMarkerLoaded() const
 {
+#ifdef QUOTIENT_07
+    const auto it = findInTimeline(lastFullyReadEventId());
+#else
     const auto it = findInTimeline(readMarkerEventId());
+#endif
     return it != historyEdge();
 }
 
@@ -845,9 +890,9 @@ QString NeoChatRoom::htmlSafeDisplayName() const
     return displayName().toHtmlEscaped();
 }
 
-void NeoChatRoom::deleteMessagesByUser(const QString &user)
+void NeoChatRoom::deleteMessagesByUser(const QString &user, const QString &reason)
 {
-    doDeleteMessagesByUser(user);
+    doDeleteMessagesByUser(user, reason);
 }
 
 QString NeoChatRoom::joinRule() const
@@ -855,7 +900,47 @@ QString NeoChatRoom::joinRule() const
     return getCurrentState<JoinRulesEvent>()->joinRule();
 }
 
-QCoro::Task<void> NeoChatRoom::doDeleteMessagesByUser(const QString &user)
+void NeoChatRoom::setJoinRule(const QString &joinRule)
+{
+    if (!canSendState("m.room.join_rules")) {
+        qWarning() << "Power level too low to set join rules";
+        return;
+    }
+#ifdef QUOTIENT_07
+    setState("m.room.join_rules", "", QJsonObject{{"join_rule", joinRule}});
+#else
+    setState<JoinRulesEvent>(QJsonObject{{"type", "m.room.join_rules"}, {"state_key", ""}, {"content", QJsonObject{{"join_rule", joinRule}}}});
+#endif
+    // Not emitting joinRuleChanged() here, since that would override the change in the UI with the *current* value, which is not the *new* value.
+}
+
+QString NeoChatRoom::historyVisibility() const
+{
+#ifdef QUOTIENT_07
+    return currentState().get("m.room.history_visibility")->contentJson()["history_visibility"_ls].toString();
+#else
+    return getCurrentState("m.room.history_visibility")->contentJson()["history_visibility"_ls].toString();
+#endif
+}
+
+void NeoChatRoom::setHistoryVisibility(const QString &historyVisibilityRule)
+{
+    if (!canSendState("m.room.history_visibility")) {
+        qWarning() << "Power level too low to set history visibility";
+        return;
+    }
+
+#ifdef QUOTIENT_07
+    setState("m.room.history_visibility", "", QJsonObject{{"history_visibility", historyVisibilityRule}});
+#else
+    qWarning() << "Quotient 0.7 required to set history visibility";
+    return;
+#endif
+
+    // Not emitting historyVisibilityChanged() here, since that would override the change in the UI with the *current* value, which is not the *new* value.
+}
+
+QCoro::Task<void> NeoChatRoom::doDeleteMessagesByUser(const QString &user, QString reason)
 {
     QStringList events;
     for (const auto &event : messageEvents()) {
@@ -864,7 +949,7 @@ QCoro::Task<void> NeoChatRoom::doDeleteMessagesByUser(const QString &user)
         }
     }
     for (const auto &e : events) {
-        auto job = connection()->callApi<RedactEventJob>(id(), QUrl::toPercentEncoding(e), connection()->generateTxnId());
+        auto job = connection()->callApi<RedactEventJob>(id(), QUrl::toPercentEncoding(e), connection()->generateTxnId(), reason);
         co_await qCoro(job, &BaseJob::finished);
         if (job->error() != BaseJob::Success) {
             qWarning() << "Error: \"" << job->error() << "\" while deleting messages. Aborting";
@@ -1090,7 +1175,196 @@ void NeoChatRoom::reportEvent(const QString &eventId, const QString &reason)
     auto job = connection()->callApi<ReportContentJob>(id(), eventId, -50, reason);
     connect(job, &BaseJob::finished, this, [this, job]() {
         if (job->error() == BaseJob::Success) {
-            Q_EMIT positiveMessage(i18n("Report sent successfully."));
+            Q_EMIT showMessage(Positive, i18n("Report sent successfully."));
+            Q_EMIT showMessage(MessageType::Positive, i18n("Report sent successfully."));
+        }
+    });
+}
+
+QString NeoChatRoom::chatBoxText() const
+{
+    return m_chatBoxText;
+}
+
+void NeoChatRoom::setChatBoxText(const QString &text)
+{
+    m_chatBoxText = text;
+    Q_EMIT chatBoxTextChanged();
+}
+
+QString NeoChatRoom::chatBoxReplyId() const
+{
+    return m_chatBoxReplyId;
+}
+
+void NeoChatRoom::setChatBoxReplyId(const QString &replyId)
+{
+    if (replyId == m_chatBoxReplyId) {
+        return;
+    }
+    m_chatBoxReplyId = replyId;
+    Q_EMIT chatBoxReplyIdChanged();
+}
+
+QString NeoChatRoom::chatBoxEditId() const
+{
+    return m_chatBoxEditId;
+}
+
+void NeoChatRoom::setChatBoxEditId(const QString &editId)
+{
+    if (editId == m_chatBoxEditId) {
+        return;
+    }
+    m_chatBoxEditId = editId;
+    Q_EMIT chatBoxEditIdChanged();
+}
+
+NeoChatUser *NeoChatRoom::chatBoxReplyUser() const
+{
+    if (m_chatBoxReplyId.isEmpty()) {
+        return nullptr;
+    }
+    return static_cast<NeoChatUser *>(user((*findInTimeline(m_chatBoxReplyId))->senderId()));
+}
+
+QString NeoChatRoom::chatBoxReplyMessage() const
+{
+    if (m_chatBoxReplyId.isEmpty()) {
+        return {};
+    }
+    return eventToString(*static_cast<const RoomMessageEvent *>(&**findInTimeline(m_chatBoxReplyId)));
+}
+
+NeoChatUser *NeoChatRoom::chatBoxEditUser() const
+{
+    if (m_chatBoxEditId.isEmpty()) {
+        return nullptr;
+    }
+    return static_cast<NeoChatUser *>(user((*findInTimeline(m_chatBoxEditId))->senderId()));
+}
+
+QString NeoChatRoom::chatBoxEditMessage() const
+{
+    if (m_chatBoxEditId.isEmpty()) {
+        return {};
+    }
+    return eventToString(*static_cast<const RoomMessageEvent *>(&**findInTimeline(m_chatBoxEditId)));
+}
+
+QString NeoChatRoom::chatBoxAttachmentPath() const
+{
+    return m_chatBoxAttachmentPath;
+}
+
+void NeoChatRoom::setChatBoxAttachmentPath(const QString &attachmentPath)
+{
+    m_chatBoxAttachmentPath = attachmentPath;
+    Q_EMIT chatBoxAttachmentPathChanged();
+}
+
+QVector<Mention> *NeoChatRoom::mentions()
+{
+    return &m_mentions;
+}
+
+QString NeoChatRoom::savedText() const
+{
+    return m_savedText;
+}
+
+void NeoChatRoom::setSavedText(const QString &savedText)
+{
+    m_savedText = savedText;
+}
+
+bool NeoChatRoom::canEncryptRoom() const
+{
+#ifdef QUOTIENT_07
+#ifdef Quotient_E2EE_ENABLED
+    return !usesEncryption() && canSendState("m.room.encryption");
+#endif
+#endif
+    return false;
+}
+
+#ifdef QUOTIENT_07
+PollHandler *NeoChatRoom::poll(const QString &eventId)
+{
+    if (!m_polls.contains(eventId)) {
+        auto handler = new PollHandler(this);
+        handler->setRoom(this);
+        handler->setPollStartEventId(eventId);
+        m_polls.insert(eventId, handler);
+    }
+    return m_polls[eventId];
+}
+#endif
+
+bool NeoChatRoom::downloadTempFile(const QString &eventId)
+{
+    QTemporaryFile file;
+    file.setAutoRemove(false);
+    if (!file.open()) {
+        return false;
+    }
+
+    download(eventId, QUrl::fromLocalFile(file.fileName()));
+    return true;
+}
+
+void NeoChatRoom::download(const QString &eventId, const QUrl &localFilename)
+{
+    downloadFile(eventId, localFilename);
+#ifndef Q_OS_ANDROID
+    auto job = new FileTransferPseudoJob(FileTransferPseudoJob::Download, localFilename.toLocalFile(), eventId);
+    connect(this, &Room::fileTransferProgress, job, &FileTransferPseudoJob::fileTransferProgress);
+    connect(this, &Room::fileTransferCompleted, job, &FileTransferPseudoJob::fileTransferCompleted);
+    connect(this, &Room::fileTransferFailed, job, &FileTransferPseudoJob::fileTransferFailed);
+    KIO::getJobTracker()->registerJob(job);
+    job->start();
+#endif
+}
+
+void NeoChatRoom::mapAlias(const QString &alias)
+{
+    auto getLocalAliasesJob = connection()->callApi<GetLocalAliasesJob>(id());
+    connect(getLocalAliasesJob, &BaseJob::success, this, [this, getLocalAliasesJob, alias] {
+        if (getLocalAliasesJob->aliases().contains(alias)) {
+            return;
+        } else {
+            auto setRoomAliasJob = connection()->callApi<SetRoomAliasJob>(alias, id());
+            connect(setRoomAliasJob, &BaseJob::success, this, [this, alias] {
+                auto newAltAliases = altAliases();
+                newAltAliases.append(alias);
+                setLocalAliases(newAltAliases);
+            });
+        }
+    });
+}
+
+void NeoChatRoom::unmapAlias(const QString &alias)
+{
+    connection()->callApi<DeleteRoomAliasJob>(alias);
+}
+
+void NeoChatRoom::setCanonicalAlias(const QString &newAlias)
+{
+    QString oldCanonicalAlias = canonicalAlias();
+    Room::setCanonicalAlias(newAlias);
+
+    connect(this, &Room::namesChanged, this, [this, newAlias, oldCanonicalAlias] {
+        if (canonicalAlias() == newAlias) {
+            // If the new canonical alias is already a published alt alias remove it otherwise it will be in both lists.
+            // The server doesn't prevent this so we need to handle it.
+            auto newAltAliases = altAliases();
+            if (!oldCanonicalAlias.isEmpty()) {
+                newAltAliases.append(oldCanonicalAlias);
+            }
+            if (newAltAliases.contains(newAlias)) {
+                newAltAliases.removeAll(newAlias);
+                Room::setLocalAliases(newAltAliases);
+            }
         }
     });
 }
