@@ -3,25 +3,17 @@
 
 #pragma once
 
-#include "joinrulesevent.h"
-#include <events/encryptionevent.h>
-#include <events/redactionevent.h>
-#include <events/roomavatarevent.h>
-#include <events/roomcreateevent.h>
-#include <events/roommemberevent.h>
-#include <events/roommessageevent.h>
-#include <events/simplestateevents.h>
+#include <qobjectdefs.h>
 #include <room.h>
 
+#include <QCache>
 #include <QObject>
-#include <QPointer>
-#include <QTimer>
+#include <QTextCursor>
 
 #include <qcoro/task.h>
 
-#include "neochatuser.h"
-
-using namespace Quotient;
+class PollHandler;
+class NeoChatUser;
 
 class PushNotificationState : public QObject
 {
@@ -38,24 +30,50 @@ public:
     Q_ENUM(State);
 };
 
-class NeoChatRoom : public Room
+struct Mention {
+    QTextCursor cursor;
+    QString text;
+    int start = 0;
+    int position = 0;
+    QString id;
+};
+
+class NeoChatRoom : public Quotient::Room
 {
     Q_OBJECT
     Q_PROPERTY(QVariantList usersTyping READ getUsersTyping NOTIFY typingChanged)
-    Q_PROPERTY(QString cachedInput MEMBER m_cachedInput NOTIFY cachedInputChanged)
     Q_PROPERTY(bool hasFileUploading READ hasFileUploading WRITE setHasFileUploading NOTIFY hasFileUploadingChanged)
     Q_PROPERTY(int fileUploadingProgress READ fileUploadingProgress NOTIFY fileUploadingProgressChanged)
     Q_PROPERTY(QString avatarMediaId READ avatarMediaId NOTIFY avatarChanged STORED false)
     Q_PROPERTY(bool readMarkerLoaded READ readMarkerLoaded NOTIFY readMarkerLoadedChanged)
     Q_PROPERTY(QDateTime lastActiveTime READ lastActiveTime NOTIFY lastActiveTimeChanged)
     Q_PROPERTY(bool isInvite READ isInvite NOTIFY isInviteChanged)
-    Q_PROPERTY(QString joinRule READ joinRule CONSTANT)
+    Q_PROPERTY(QString joinRule READ joinRule WRITE setJoinRule NOTIFY joinRuleChanged)
+    Q_PROPERTY(QString historyVisibility READ historyVisibility WRITE setHistoryVisibility NOTIFY historyVisibilityChanged)
     Q_PROPERTY(QString htmlSafeDisplayName READ htmlSafeDisplayName NOTIFY displayNameChanged)
     Q_PROPERTY(PushNotificationState::State pushNotificationState MEMBER m_currentPushNotificationState WRITE setPushNotificationState NOTIFY
                    pushNotificationStateChanged)
 
+    // Due to problems with QTextDocument, unlike the other properties here, chatBoxText is *not* used to store the text when switching rooms
+    Q_PROPERTY(QString chatBoxText READ chatBoxText WRITE setChatBoxText NOTIFY chatBoxTextChanged)
+    Q_PROPERTY(QString chatBoxReplyId READ chatBoxReplyId WRITE setChatBoxReplyId NOTIFY chatBoxReplyIdChanged)
+    Q_PROPERTY(QString chatBoxEditId READ chatBoxEditId WRITE setChatBoxEditId NOTIFY chatBoxEditIdChanged)
+    Q_PROPERTY(NeoChatUser *chatBoxReplyUser READ chatBoxReplyUser NOTIFY chatBoxReplyIdChanged)
+    Q_PROPERTY(QString chatBoxReplyMessage READ chatBoxReplyMessage NOTIFY chatBoxReplyIdChanged)
+    Q_PROPERTY(NeoChatUser *chatBoxEditUser READ chatBoxEditUser NOTIFY chatBoxEditIdChanged)
+    Q_PROPERTY(QString chatBoxEditMessage READ chatBoxEditMessage NOTIFY chatBoxEditIdChanged)
+    Q_PROPERTY(QString chatBoxAttachmentPath READ chatBoxAttachmentPath WRITE setChatBoxAttachmentPath NOTIFY chatBoxAttachmentPathChanged)
+    Q_PROPERTY(bool canEncryptRoom READ canEncryptRoom NOTIFY canEncryptRoomChanged)
+
 public:
-    explicit NeoChatRoom(Connection *connection, QString roomId, JoinState joinState = {});
+    enum MessageType {
+        Positive,
+        Info,
+        Error,
+    };
+    Q_ENUM(MessageType);
+
+    explicit NeoChatRoom(Quotient::Connection *connection, QString roomId, Quotient::JoinState joinState = {});
 
     [[nodiscard]] QVariantList getUsersTyping() const;
 
@@ -64,7 +82,7 @@ public:
     /// This function respect the showLeaveJoinEvent setting and discard
     /// other not interesting events. This function can return an empty pointer
     /// when the room is empty of RoomMessageEvent.
-    [[nodiscard]] const RoomMessageEvent *lastEvent(bool ignoreStateEvent = false) const;
+    [[nodiscard]] const Quotient::RoomEvent *lastEvent(bool ignoreStateEvent = false) const;
 
     /// Convenient way to get the last event but in a string format.
     ///
@@ -94,6 +112,10 @@ public:
     bool isEventHighlighted(const Quotient::RoomEvent *e) const;
 
     [[nodiscard]] QString joinRule() const;
+    void setJoinRule(const QString &joinRule);
+
+    [[nodiscard]] QString historyVisibility() const;
+    void setHistoryVisibility(const QString &historyVisibilityRule);
 
     [[nodiscard]] bool hasFileUploading() const
     {
@@ -134,7 +156,7 @@ public:
 
     [[nodiscard]] QString avatarMediaId() const;
 
-    [[nodiscard]] QString eventToString(const RoomEvent &evt, Qt::TextFormat format = Qt::PlainText, bool removeReply = true) const;
+    [[nodiscard]] QString eventToString(const Quotient::RoomEvent &evt, Qt::TextFormat format = Qt::PlainText, bool removeReply = true) const;
 
     Q_INVOKABLE [[nodiscard]] bool containsUser(const QString &userID) const;
     Q_INVOKABLE [[nodiscard]] bool isUserBanned(const QString &user) const;
@@ -151,6 +173,49 @@ public:
 
     Q_INVOKABLE void setPushNotificationState(PushNotificationState::State state);
 
+    Q_INVOKABLE void download(const QString &eventId, const QUrl &localFilename = {});
+
+    QString chatBoxText() const;
+    void setChatBoxText(const QString &text);
+
+    QString chatBoxReplyId() const;
+    void setChatBoxReplyId(const QString &replyId);
+
+    NeoChatUser *chatBoxReplyUser() const;
+    QString chatBoxReplyMessage() const;
+
+    QString chatBoxEditId() const;
+    void setChatBoxEditId(const QString &editId);
+
+    NeoChatUser *chatBoxEditUser() const;
+    QString chatBoxEditMessage() const;
+
+    QString chatBoxAttachmentPath() const;
+    void setChatBoxAttachmentPath(const QString &attachmentPath);
+
+    QVector<Mention> *mentions();
+
+    QString savedText() const;
+    void setSavedText(const QString &savedText);
+
+    bool canEncryptRoom() const;
+
+    Q_INVOKABLE bool downloadTempFile(const QString &eventId);
+
+    /*
+     * Map an alias to the room
+     *
+     * Note: this is different to setLocalAliases as that can only
+     * get the room to publish and alias that is already mapped.
+     */
+    Q_INVOKABLE void mapAlias(const QString &alias);
+    Q_INVOKABLE void unmapAlias(const QString &alias);
+    Q_INVOKABLE void setCanonicalAlias(const QString &newAlias);
+
+#ifdef QUOTIENT_07
+    Q_INVOKABLE PollHandler *poll(const QString &eventId);
+#endif
+
 #ifndef QUOTIENT_07
     Q_INVOKABLE QString htmlSafeMemberName(const QString &userId) const
     {
@@ -159,25 +224,33 @@ public:
 #endif
 
 private:
-    QString m_cachedInput;
     QSet<const Quotient::RoomEvent *> highlights;
 
     bool m_hasFileUploading = false;
     int m_fileUploadingProgress = 0;
 
-    PushNotificationState::State m_currentPushNotificationState = PushNotificationState::State::Unknown;
+    PushNotificationState::State m_currentPushNotificationState = PushNotificationState::Unknown;
     bool m_pushNotificationStateUpdating = false;
 
     void checkForHighlights(const Quotient::TimelineItem &ti);
 
     void onAddNewTimelineEvents(timeline_iter_t from) override;
     void onAddHistoricalTimelineEvents(rev_iter_t from) override;
-    void onRedaction(const RoomEvent &prevEvent, const RoomEvent &after) override;
+    void onRedaction(const Quotient::RoomEvent &prevEvent, const Quotient::RoomEvent &after) override;
 
-    static QString markdownToHTML(const QString &markdown);
-    QCoro::Task<void> doDeleteMessagesByUser(const QString &user);
+    QCoro::Task<void> doDeleteMessagesByUser(const QString &user, QString reason);
     QCoro::Task<void> doUploadFile(QUrl url, QString body = QString());
 
+    QString m_chatBoxText;
+    QString m_chatBoxReplyId;
+    QString m_chatBoxEditId;
+    QString m_chatBoxAttachmentPath;
+    QVector<Mention> m_mentions;
+    QString m_savedText;
+#ifdef QUOTIENT_07
+    QCache<QString, PollHandler> m_polls;
+#endif
+
 private Q_SLOTS:
     void countChanged();
     void updatePushNotificationState(QString type);
@@ -193,14 +266,20 @@ Q_SIGNALS:
     void isInviteChanged();
     void displayNameChanged();
     void pushNotificationStateChanged(PushNotificationState::State state);
-    void positiveMessage(const QString &message);
+    void showMessage(MessageType messageType, const QString &message);
+    void chatBoxTextChanged();
+    void chatBoxReplyIdChanged();
+    void chatBoxEditIdChanged();
+    void chatBoxAttachmentPathChanged();
+    void canEncryptRoomChanged();
+    void joinRuleChanged();
+    void historyVisibilityChanged();
 
 public Q_SLOTS:
     void uploadFile(const QUrl &url, const QString &body = QString());
     void acceptInvitation();
     void forget();
     void sendTypingNotification(bool isTyping);
-    QString preprocessText(const QString &text);
 
     /// @param rawText The text as it was typed.
     /// @param cleanedText The text with link to the users.
@@ -218,5 +297,5 @@ public Q_SLOTS:
     void addLocalAlias(const QString &alias);
     void removeLocalAlias(const QString &alias);
     void toggleReaction(const QString &eventId, const QString &reaction);
-    void deleteMessagesByUser(const QString &user);
+    void deleteMessagesByUser(const QString &user, const QString &reason);
 };
