@@ -6,13 +6,12 @@
 #include "controller.h"
 #include "neochatconfig.h"
 #include "neochatroom.h"
-#include <KConfigGroup>
 #include <KLocalizedString>
 #include <QDesktopServices>
 #include <QStandardPaths>
-#include <csapi/joining.h>
 #include <qt_connection_util.h>
-#include <utility>
+#include <QQuickTextDocument>
+#include <user.h>
 
 #ifndef Q_OS_ANDROID
 #include <KIO/OpenUrlJob>
@@ -125,6 +124,12 @@ void RoomManager::openRoomForActiveConnection()
 
 void RoomManager::enterRoom(NeoChatRoom *room)
 {
+    if (m_chatDocumentHandler) {
+        // We're doing these things here because it is critical that they are switched at the same time
+        m_currentRoom->setSavedText(m_chatDocumentHandler->document()->textDocument()->toPlainText());
+        m_chatDocumentHandler->setRoom(room);
+        m_chatDocumentHandler->document()->textDocument()->setPlainText(room->savedText());
+    }
     m_lastCurrentRoom = std::exchange(m_currentRoom, room);
     Q_EMIT currentRoomChanged();
 
@@ -234,3 +239,15 @@ void RoomManager::leaveRoom(NeoChatRoom *room)
 
     room->forget();
 }
+
+ChatDocumentHandler *RoomManager::chatDocumentHandler() const
+{
+    return m_chatDocumentHandler;
+}
+
+void RoomManager::setChatDocumentHandler(ChatDocumentHandler *handler)
+{
+    m_chatDocumentHandler = handler;
+    m_chatDocumentHandler->setRoom(m_currentRoom);
+    Q_EMIT chatDocumentHandlerChanged();
+}
